searchData={"items":[{"type":"module","title":"Scurry.Astar","doc":"Implementation of [A-star search\nalgorithm](https://en.wikipedia.org/wiki/A*_search_algorithm) to find the\nshortest path in graphs or 2D polygon maps.\n\nBasic usage is;\n\n```\n# Define function for the heuristic cost between two nodes\ndef heur_fun(node_a, node_b) do\n  ... returns the cost of node a to node b\nend\n\n# Define a graph\ngraph = %{\n  node_1 => [\n    {node_2, cost_1_to_2}, {node_3, cost_1_to_3},\n  ],\n  node_2 => [\n    {node_3, cost_2_to_3}, {node_4, cost_2_to_4},\n  ],\n  ...\n}\n\n# Do A-star search\nstate = Astar.search(graph, node_1, node_z, &heur_fun/2)\n\n# Extract path from the state\npath = Astar.path(state)\n[node1, ..., node_z]\n```\n\nSee [Quickstart](quickstart.html) for a concrete end-to-end example of\ndefining a world and holes/obstacles, then using `PolygonMap` and `Astar`\nmodules to do path finding within this.","ref":"Scurry.Astar.html"},{"type":"type","title":"Scurry.Astar.cost/0","doc":"The cost of a traversel between two nodes is a numeric value","ref":"Scurry.Astar.html#t:cost/0"},{"type":"type","title":"Scurry.Astar.cost_fun/0","doc":"Function that given two `t:gnode/0` graph nodes, computes the cost. Eg. a euclidian 2D vector function like `Scurry.Vector.distance/2`. `(gnode, gnode -> cost)`","ref":"Scurry.Astar.html#t:cost_fun/0"},{"type":"type","title":"Scurry.Astar.gnode/0","doc":"A graph node (spelled `gnode` since `node` is reserved) is any type that can be used as a key in a `t:map/0` For instance a `t:vector/0`.","ref":"Scurry.Astar.html#t:gnode/0"},{"type":"type","title":"Scurry.Astar.graph/0","doc":"A graph is a map from `t:gnode/0` to a list of `t:gnode/0` and the traversal `t:cost/0`, `%{gnode => [{gnode, cost}, ...]}`","ref":"Scurry.Astar.html#t:graph/0"},{"type":"function","title":"Scurry.Astar.path/1","doc":"Get the path from the state returned by `search/4`.","ref":"Scurry.Astar.html#path/1"},{"type":"function","title":"Params - Scurry.Astar.path/1","doc":"* `state` the a-star state returned by `search/4`","ref":"Scurry.Astar.html#path/1-params"},{"type":"function","title":"Returns - Scurry.Astar.path/1","doc":"The path as a list of nodes. The type of the node is the same as in the\n`t:graph/0` used in the call to `search/4`.","ref":"Scurry.Astar.html#path/1-returns"},{"type":"function","title":"Scurry.Astar.search/4","doc":"Find shortest path in `graph` from `start` to `stop`.","ref":"Scurry.Astar.html#search/4"},{"type":"function","title":"Params - Scurry.Astar.search/4","doc":"* `graph` a map of node to a list of nodes and cost tuples.\n* `start` the node from which to start\n* `stop` the node at which to end\n* `heur_fun` the heuristic function used to estimate cost from a node in\n  `graph` to `stop`. It takes two nodes and returns a cost that should be\n  comparable with itself for ordering. `node, node :: term`.","ref":"Scurry.Astar.html#search/4-params"},{"type":"function","title":"Returns - Scurry.Astar.search/4","doc":"The algorithms internal state which can be passed to `path/1` to\nobtain the actual path.","ref":"Scurry.Astar.html#search/4-returns"},{"type":"opaque","title":"Scurry.Astar.state/0","doc":"The internal state of the A-star algorithm. Use `path/1` to extract the result.","ref":"Scurry.Astar.html#t:state/0"},{"type":"module","title":"Scurry.Geo","doc":"Geometry functions related to lines relevant for 2D map pathfinding.","ref":"Scurry.Geo.html"},{"type":"function","title":"Scurry.Geo.distance_to_segment/2","doc":"Get the distance from a point to a line/segment.\n\nThis is  the square root of `distance_squared/2`.","ref":"Scurry.Geo.html#distance_to_segment/2"},{"type":"function","title":"Params - Scurry.Geo.distance_to_segment/2","doc":"* `line` (`t:line/0`) the line to compute the distance to.\n* `point` (`t:vector/0`) the point to compute the distance to `line` for.","ref":"Scurry.Geo.html#distance_to_segment/2-params"},{"type":"function","title":"Returns - Scurry.Geo.distance_to_segment/2","doc":"The distance beween the given `point` and `line`.","ref":"Scurry.Geo.html#distance_to_segment/2-returns"},{"type":"function","title":"Examples - Scurry.Geo.distance_to_segment/2","doc":"iex> Geo.distance_to_segment({{2, 0}, {2, 2}}, {0, 1})\n    2.0","ref":"Scurry.Geo.html#distance_to_segment/2-examples"},{"type":"function","title":"Scurry.Geo.distance_to_segment_squared/2","doc":"Get the distance squared from `point` to `line`.","ref":"Scurry.Geo.html#distance_to_segment_squared/2"},{"type":"function","title":"Params - Scurry.Geo.distance_to_segment_squared/2","doc":"* `line` (`t:line/0`) the line to compute the distance squared to.\n* `point` (`t:vector/0`) the point to compute the distance squared to `line` for.","ref":"Scurry.Geo.html#distance_to_segment_squared/2-params"},{"type":"function","title":"Returns - Scurry.Geo.distance_to_segment_squared/2","doc":"The square of the distance beween the given `point` and `line`.","ref":"Scurry.Geo.html#distance_to_segment_squared/2-returns"},{"type":"function","title":"Examples - Scurry.Geo.distance_to_segment_squared/2","doc":"iex> Geo.distance_to_segment_squared({{2, 0}, {2, 2}}, {0, 1})\n    4.0","ref":"Scurry.Geo.html#distance_to_segment_squared/2-examples"},{"type":"function","title":"Scurry.Geo.do_lines_intersect?/2","doc":"Check if two lines intersect\n\nThis is a simpler version of `line_segment_intersection/2`, which is typically\na better choice since it handles endpoints and segment overlap too.\n\nThis is kept/provided for historic purposes.","ref":"Scurry.Geo.html#do_lines_intersect?/2"},{"type":"function","title":"Params - Scurry.Geo.do_lines_intersect?/2","doc":"* `line1` (`t:line/0`) a line to check for intersection with\n* `line2` (`t:line/0`) a line to check if it intersects with `line1`.","ref":"Scurry.Geo.html#do_lines_intersect?/2-params"},{"type":"function","title":"Returns - Scurry.Geo.do_lines_intersect?/2","doc":"`true` if they intersect, `false` otherwise.\n\n>","ref":"Scurry.Geo.html#do_lines_intersect?/2-returns"},{"type":"function","title":"Note - Scurry.Geo.do_lines_intersect?/2","doc":">\n> This doesn't handle segment overlap or points touching. Use\n> `line_segment_intersection/2` instead for that level of detail.","ref":"Scurry.Geo.html#do_lines_intersect?/2-note"},{"type":"type","title":"Scurry.Geo.line/0","doc":"A line is represented by two vertices of type `t:vector/0`","ref":"Scurry.Geo.html#t:line/0"},{"type":"function","title":"Scurry.Geo.line_segment_intersection/2","doc":"Determine if, where and how two lines intersect.\n\nThis function computes whether two lines intersect (cross each other) touch\non an endpoint or are on top of each other.","ref":"Scurry.Geo.html#line_segment_intersection/2"},{"type":"function","title":"Params - Scurry.Geo.line_segment_intersection/2","doc":"* `line1` (`t:line/0`) a line segment\n* `line2` (`t:line/0`) a line segment to check where and how it intersects with `line1`.","ref":"Scurry.Geo.html#line_segment_intersection/2-params"},{"type":"function","title":"Returns - Scurry.Geo.line_segment_intersection/2","doc":"* `:none` no intersection.\n* `:parallel` the lines are parallel and _do not_ intersect.\n* `:on_segment` one line is on the other. They may extend past each other's endpoints.\n* `{:point_intersection, vector}` either line has an _endpoint_ on the other\n  line. This means they just touch at the returned vector.\n* `{:intersection, vector}` the lines intersect at the returned vector.","ref":"Scurry.Geo.html#line_segment_intersection/2-returns"},{"type":"function","title":"Examples - Scurry.Geo.line_segment_intersection/2","doc":"iex> Geo.line_segment_intersection({{1, 2}, {4, 2}}, {{2, 0}, {3, 0}})\n    :parallel\n    iex> Geo.line_segment_intersection({{1, 2}, {4, 2}}, {{2, 2}, {3, 2}})\n    :on_segment\n    iex> Geo.line_segment_intersection({{1, 2}, {4, 2}}, {{2, 2}, {5, 2}})\n    :on_segment\n    iex> Geo.line_segment_intersection({{1, 2}, {4, 2}}, {{2, 0}, {2, 1}})\n    :none\n    iex> Geo.line_segment_intersection({{1, 2}, {4, 2}}, {{2, 0}, {2, 2}})\n    {:point_intersection, {2.0, 2.0}}\n    iex> Geo.line_segment_intersection({{1, 2}, {4, 2}}, {{2, 0}, {2, 3}})\n    {:intersection, {2.0, 2.0}}","ref":"Scurry.Geo.html#line_segment_intersection/2-examples"},{"type":"type","title":"Scurry.Geo.polygon/0","doc":"A polygon is represented by a list of `t:vector/0`, each being a tuple of `{x, y}` coordinates, both `t:number/0`. To be properly defined, the vertices of the polygon must be non-closed and clockwise.","ref":"Scurry.Geo.html#t:polygon/0"},{"type":"type","title":"Scurry.Geo.vector/0","doc":"A vector is represented as a tuple `{x, y}`, it's x and y components, both `t:number/0`. These are used for vector math in `Scurry.Vector` and as 2D points in `Scurry.Polygon` and `Scurry.PolygonMap`.","ref":"Scurry.Geo.html#t:vector/0"},{"type":"module","title":"Scurry.Polygon","doc":"Functions related to polygons and lines relevant for 2D map pathfinding.\n\nThis provides functions for;\n\n* line of sight between two points\n* classify polygon vertices as concave or vertex\n* intersections of lines and polygons\n* checking if points are inside/outside polygons\n* finding nearest point on a polygon and distances\n\nPolygons are\n* A list of vertices, `[{x1, y1}, {x2,y2}, ...]` defined as a `list(vector)`.\n* They **must not** be closed, ie. last vertex should not be equal to the first.\n* They **must be** in clockwise order in screen coordinates, otherwise\n  convex/concave classification will be inversed as it traverses the egdes.\n\n>","ref":"Scurry.Polygon.html"},{"type":"module","title":"Order of vertices {: .warning} - Scurry.Polygon","doc":">\n> Polygons **must** be defined in clockwise order in screen coordinates. This is necessary\n> for the convex/concave classification. It counter clock, it  will be inversed as it\n> traverses the egdes.\n>\n> Here's a crude drawing of an example of the M shaped polygon used for many tests/docs.\n>\n> `polygon = [{0, 0}, {10, 0}, {20, 0}, {20, 20}, {10, 10}, {0, 20}]`\n> ![Order of vertices](imgs/polygon.png)","ref":"Scurry.Polygon.html#module-order-of-vertices-warning"},{"type":"function","title":"Scurry.Polygon.classify_vertex/2","doc":"Check if a specific vertex is concave, convex or neither.\n\nWhehter a vertex is concave or convex is defined by it pointing out - it's\ninner angle is less than 180 means convex and more than 180 means concave.\n\nWhen testing a vertex, keep this in mind and negate appropriately depending\non whether it's the boundary polygon or a hole polygon being tested.","ref":"Scurry.Polygon.html#classify_vertex/2"},{"type":"function","title":"Params - Scurry.Polygon.classify_vertex/2","doc":"* `polygon` (`t:polygon/0`) the polygon in which to check a vertex.\n* `at` (`t:integer/0`), which vertex in the `polygon` to check.","ref":"Scurry.Polygon.html#classify_vertex/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.classify_vertex/2","doc":"* `:convex` for a convex vertice.\n* `:concave` for a concave vertice.\n* `:neither` for a vertice that's a straight edge, ie. 180 degrees.","ref":"Scurry.Polygon.html#classify_vertex/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.classify_vertex/2","doc":"# A vaguely M shaped polygon\n    iex> m = [{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}]\n    [{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}]\n    iex> Polygon.classify_vertex(m, 0)\n    :convex\n    iex> Polygon.classify_vertex(m, 1)\n    :neither\n    iex> Polygon.classify_vertex(m, 4)\n    :concave","ref":"Scurry.Polygon.html#classify_vertex/2-examples"},{"type":"function","title":"Scurry.Polygon.classify_vertices/1","doc":"Split a polygon into concave and convex vertices.\n\nWhen doing pathfinding, there will typically be a outer polygon bounding the\n\"world\" and multiple inner polygons describing \"holes/obstacles\".\n\nThe path can only be within the outer polygon and has to \"walk around\" the\nholes.\n\nClassifying the polygons into concave and convex is used to determine the\nwalkable graph.\n\n* The outer polygon's concave (pointing into the world) vertices are used.\n  used.\n* The holes' convex (point out of the hole, into the world) vertices used used.\n\nIn code, this looks like\n\n```\n# Get the concave vectices of the bounding polygon\n{concave, _convex} = Polygon.classify_vertices(world)\n\n# Get al lthe convex vertices of all the holes\nconvex = Enum.reduce(holes, [], fn points, acc ->\n  {_, convex} = Polygon.classify_vertices(points)\n  acc ++ convex\nend)\n\n# The initial walk map is the combined set\nvertices = concave ++ convex\n```","ref":"Scurry.Polygon.html#classify_vertices/1"},{"type":"function","title":"Params - Scurry.Polygon.classify_vertices/1","doc":"* `polygon` (`t:polygon/0`) the polygon to classify.","ref":"Scurry.Polygon.html#classify_vertices/1-params"},{"type":"function","title":"Returns - Scurry.Polygon.classify_vertices/1","doc":"`{list of concave vertices, list of convex}`.\n\n* A tuple of two lists of `t:vector/0`.\n* The first list are all the _concave_ vertices or `[]` if none\n* The second list are all the _convex_ vertices or `[]` if none","ref":"Scurry.Polygon.html#classify_vertices/1-returns"},{"type":"function","title":"Note - Scurry.Polygon.classify_vertices/1","doc":"Three points that fall on the same line (`[{0, 0}, {1, 0}, {2, 0}]`) does not\nmatch neither the concave/convex definition (angle greater-than / less-than\n180 degrees) this function will discard these via `classify_vertex/2`.","ref":"Scurry.Polygon.html#classify_vertices/1-note"},{"type":"function","title":"Examples - Scurry.Polygon.classify_vertices/1","doc":"# A vaguely M shaped polygon\n    iex> Polygon.classify_vertices([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}])\n    {[{1, 0.5}], [{0, 0}, {2, 0}, {2, 1}, {0, 1}]}\n\n    # A square around 0,0\n    iex> Polygon.classify_vertices([{-1, -1}, {1, -1}, {1, 1}, {-1, 1}])\n    {[], [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]}\n\n    # A flat line\n    iex> Polygon.classify_vertices([{0, 0}, {1, 0}, {2, 0}])\n    {[], []}","ref":"Scurry.Polygon.html#classify_vertices/1-examples"},{"type":"function","title":"Scurry.Polygon.first_intersection/2","doc":"Get first intersection of a line with a polygon.\n\nThe \"opposite\" of `last_intersection/2`.","ref":"Scurry.Polygon.html#first_intersection/2"},{"type":"function","title":"Params - Scurry.Polygon.first_intersection/2","doc":"* `polygon` (`t:polygon/0`) a polygon to check for an intersection with `line`.\n* `line` (`t:line/0`). The line to check for intersections with `polygon`.\n\nA `t:line/0` is a tuple of two `t:vector/0` and the first is considered the head\nof the line and \"first\" in this context means nearest to that point.","ref":"Scurry.Polygon.html#first_intersection/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.first_intersection/2","doc":"* A `t:vector/0` indicating where `line` first intersects `polygon`.\n* `nil` if there's no intersection.","ref":"Scurry.Polygon.html#first_intersection/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.first_intersection/2","doc":"# A square around 0,0\n    iex> square = [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    iex> Polygon.first_intersection(square, {{0, -2}, {0, 2}})\n    {0.0, -1.0}\n    iex> Polygon.first_intersection(square, {{0, 2}, {0, -2}})\n    {0.0, 1.0}\n    iex> Polygon.first_intersection(square, {{2, -2}, {2, 2}})\n    nil","ref":"Scurry.Polygon.html#first_intersection/2-examples"},{"type":"function","title":"Scurry.Polygon.intersections/3","doc":"Get all intersections of a line with a polygon including their type.\n\nThis function basically calls `Geo.line_segment_intersection/2` for every segment\nof the `polygon` against the `line` and filters the results to only include\nthe list of intersection points.","ref":"Scurry.Polygon.html#intersections/3"},{"type":"function","title":"Params - Scurry.Polygon.intersections/3","doc":"* `polygon` (`t:polygon/0`), the polygon to check.\n* `line` (`t:line/0`), the line check for intersections against `polygon`.","ref":"Scurry.Polygon.html#intersections/3-params"},{"type":"function","title":"Options - Scurry.Polygon.intersections/3","doc":"* `:allow_points` (default `false`) whether a `on_point` intersection\nshould be considered an intersection or not. This varies from use\ncases. Eg. when building a polygon, points will be connected and thus\nintersect if `true`. This may not be the desired result, so `false` won't\nconsider points intersections.","ref":"Scurry.Polygon.html#intersections/3-options"},{"type":"function","title":"Returns - Scurry.Polygon.intersections/3","doc":"* a list of `t:vector/0` indicating where the line intersects\n* `[]` if there's no intersections.","ref":"Scurry.Polygon.html#intersections/3-returns"},{"type":"function","title":"Examples - Scurry.Polygon.intersections/3","doc":"iex> polygon = [{0, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}]\n    [{0, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}]\n    iex> line = {{1, -1}, {1, 3}}\n    {{1, -1}, {1, 3}}\n    iex> Polygon.intersections(polygon, line)\n    [{1.0, 0.0}]\n    iex> Polygon.intersections(polygon, line, allow_points: true)\n    [{1.0, 0.0}, {1.0, 0.5}]","ref":"Scurry.Polygon.html#intersections/3-examples"},{"type":"function","title":"Scurry.Polygon.intersects/2","doc":"Checks if a line intersects a polygon.\n\nFind an intersection (if any) between `polygon` and `line`. For most cases\nusing `intersections/3` will be a better choice to get intersections.","ref":"Scurry.Polygon.html#intersects/2"},{"type":"function","title":"Params - Scurry.Polygon.intersects/2","doc":"* `polygon` (`t:polygon/0`), the polygon to check.\n* `line` (`t:line/0`), the line to check if it intersects `polygon`.","ref":"Scurry.Polygon.html#intersects/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.intersects/2","doc":"This relates to `Scurry.Geo.line_segment_intersection/2`.\n\n- `:nointersection`, `line` does not intersect `polygon`\n- `{:point_intersection, vector}`\n- `{;intersection, line}`\n- `:on_segment`","ref":"Scurry.Polygon.html#intersects/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.intersects/2","doc":"# A vaguely M shaped polygon\n    iex> Polygon.intersects([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}], {{1, -0.5}, {1, 0.5}})\n    {:point_intersection, {1.0, 0.0}}\n    iex> Polygon.intersects([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}], {{0.5, -0.5}, {0.5, 0.5}})\n    {:intersection, {0.5, 0.0}}\n    iex> Polygon.intersects([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}], {{0.5, -1}, {0.5, 0}})\n    {:point_intersection, {0.5, 0.0}}\n    iex> Polygon.intersects([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}], {{0.5, -2}, {0.5, -1}})\n    :nointersection\n    iex> Polygon.intersects([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}], {{2, 0.1}, {2, 0.9}})\n    :on_segment","ref":"Scurry.Polygon.html#intersects/2-examples"},{"type":"function","title":"Scurry.Polygon.intersects?/2","doc":"Checks if a line intersects a polygon.\n\nThis is a boolean version of `intersects/2`. For most cases using\n`intersections/3` will be a better choice to get intersections.","ref":"Scurry.Polygon.html#intersects?/2"},{"type":"function","title":"Params - Scurry.Polygon.intersects?/2","doc":"* `polygon` (`t:polygon/0`), the polygon to check.\n* `line` (`t:line/0`), the line to check if it intersects `polygon`.","ref":"Scurry.Polygon.html#intersects?/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.intersects?/2","doc":"`true` or `false` wether the line intersects the polygon or not.","ref":"Scurry.Polygon.html#intersects?/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.intersects?/2","doc":"# A vaguely M shaped polygon\n    iex> Polygon.intersects?([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}], {{1, -0.5}, {1, 0.5}})\n    true\n    iex> Polygon.intersects?([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}], {{1, -1}, {1, 0}})\n    true\n    iex> Polygon.intersects?([{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}], {{1, -2}, {1, -1}})\n    false","ref":"Scurry.Polygon.html#intersects?/2-examples"},{"type":"function","title":"Scurry.Polygon.is_clockwise?/1","doc":"Check if the polygon is clockwise within screen coordinates.\n\nThis is provided as a utility to check polygons eg. if loading them from data\nfiles. The library internally _does not_ use this to validate polygons to\navoid a runtime penalty.","ref":"Scurry.Polygon.html#is_clockwise?/1"},{"type":"function","title":"Params - Scurry.Polygon.is_clockwise?/1","doc":"* `polygon` (`t:polygon/0`) the polygon to check to check.","ref":"Scurry.Polygon.html#is_clockwise?/1-params"},{"type":"function","title":"Returrns - Scurry.Polygon.is_clockwise?/1","doc":"`true` if the points in the polygon are defined in a clockwise orientation.","ref":"Scurry.Polygon.html#is_clockwise?/1-returrns"},{"type":"function","title":"Examples - Scurry.Polygon.is_clockwise?/1","doc":"iex> Polygon.is_clockwise?([{0, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}])\n    true\n    iex> Polygon.is_clockwise?([{0, 0}, {0, 1}, {1, 0.5}, {2, 1}, {2, 0}])\n    false","ref":"Scurry.Polygon.html#is_clockwise?/1-examples"},{"type":"function","title":"Scurry.Polygon.is_concave?/2","doc":"Check if a specific vertex in a polygon is concave","ref":"Scurry.Polygon.html#is_concave?/2"},{"type":"function","title":"Params - Scurry.Polygon.is_concave?/2","doc":"* `polygon` (`t:polygon/0`) the polygon in which to check a vertex.\n* `at` (`t:integer/0`), which vertex in the `polygon` to check.","ref":"Scurry.Polygon.html#is_concave?/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.is_concave?/2","doc":"`true` if the vertex is concave, otherwise `false`\n\nSee also `classify_vertex/2`.","ref":"Scurry.Polygon.html#is_concave?/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.is_concave?/2","doc":"# A vaguely M shaped polygon\n    iex> m = [{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}]\n    [{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}]\n    iex> Polygon.is_concave?(m, 0)\n    false\n    iex> Polygon.is_concave?(m, 1)\n    false\n    iex> Polygon.is_concave?(m, 2)\n    false\n    iex> Polygon.is_concave?(m, 3)\n    false\n    iex> Polygon.is_concave?(m, 4)\n    true\n    iex> Polygon.is_concave?(m, 5)\n    false","ref":"Scurry.Polygon.html#is_concave?/2-examples"},{"type":"function","title":"Scurry.Polygon.is_convex?/2","doc":"Check if a specific vertex in a polygon is convex","ref":"Scurry.Polygon.html#is_convex?/2"},{"type":"function","title":"Params - Scurry.Polygon.is_convex?/2","doc":"* `polygon` (`t:polygon/0`) the polygon in which to check a vertex.\n* `at` (`t:integer/0`), which vertex in the `polygon` to check.","ref":"Scurry.Polygon.html#is_convex?/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.is_convex?/2","doc":"`true` if the vertex is convex, otherwise `false`\n\nSee also `classify_vertex/2`.","ref":"Scurry.Polygon.html#is_convex?/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.is_convex?/2","doc":"# A vaguely M shaped polygon\n    iex> m = [{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}]\n    [{0, 0}, {1, 0}, {2, 0}, {2, 1}, {1, 0.5}, {0, 1}]\n    iex> Polygon.is_convex?(m, 0)\n    true\n    iex> Polygon.is_convex?(m, 1)\n    false\n    iex> Polygon.is_convex?(m, 2)\n    true\n    iex> Polygon.is_convex?(m, 3)\n    true\n    iex> Polygon.is_convex?(m, 4)\n    false\n    iex> Polygon.is_convex?(m, 5)\n    true","ref":"Scurry.Polygon.html#is_convex?/2-examples"},{"type":"function","title":"Scurry.Polygon.is_inside?/3","doc":"Check if a point is inside a polygon or not.","ref":"Scurry.Polygon.html#is_inside?/3"},{"type":"function","title":"Params - Scurry.Polygon.is_inside?/3","doc":"* `polygon` (`t:polygon/0`) the polygon to check against.\n* `point` (`t:vector/0`) the point to check if is inside (or on) `polygon`.","ref":"Scurry.Polygon.html#is_inside?/3-params"},{"type":"function","title":"Options - Scurry.Polygon.is_inside?/3","doc":"* `allow_border`, defaults to `true` and allows `point` to be the `polygon` edges.","ref":"Scurry.Polygon.html#is_inside?/3-options"},{"type":"function","title":"Returns - Scurry.Polygon.is_inside?/3","doc":"* `true` is the `point` is inside the `polygon`\n* `true` is the `point` is on `polygon` edges if `allow_border` is `true`.","ref":"Scurry.Polygon.html#is_inside?/3-returns"},{"type":"function","title":"Examples - Scurry.Polygon.is_inside?/3","doc":"# A square around 0,0\n    iex> square = [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    iex> Polygon.is_inside?(square, {0, 0})\n    true\n    iex> Polygon.is_inside?(square, {-1, 0})\n    true\n    iex> Polygon.is_inside?(square, {-1, 0}, [allow_border: false])\n    false\n    iex> Polygon.is_inside?(square, {-2, 0})\n    false","ref":"Scurry.Polygon.html#is_inside?/3-examples"},{"type":"function","title":"Scurry.Polygon.is_outside?/3","doc":"The opposite of `is_inside?/3`, provided for code readability.\n\nSee `is_inside?/3` for behaviour and imagine the opposite.","ref":"Scurry.Polygon.html#is_outside?/3"},{"type":"function","title":"Scurry.Polygon.last_intersection/2","doc":"Get last intersection of a line with a polygon.\n\nThe \"opposite\" of `first_intersection/2`.","ref":"Scurry.Polygon.html#last_intersection/2"},{"type":"function","title":"Params - Scurry.Polygon.last_intersection/2","doc":"* `polygon` (`t:polygon/0`) a polygon to check for an intersection with `line`.\n* `line` (`t:line/0`). The line to check for intersections with `polygon`.\n\nA `t:line/0` is a tuple of two `t:vector/0` and the last is considered the tail\nof the line and \"last\" in this context means nearest to that point.","ref":"Scurry.Polygon.html#last_intersection/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.last_intersection/2","doc":"* A `t:vector/0` indicating where `line` last intersects `polygon`\n* `nil` if there's no intersection.","ref":"Scurry.Polygon.html#last_intersection/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.last_intersection/2","doc":"# A square around 0,0\n    iex> square = [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    iex> Polygon.last_intersection(square, {{0, -2}, {0, 2}})\n    {0.0, 1.0}\n    iex> Polygon.last_intersection(square, {{0, 2}, {0, -2}})\n    {0.0, -1.0}\n    iex> Polygon.last_intersection(square, {{2, -2}, {2, 2}})\n    nil","ref":"Scurry.Polygon.html#last_intersection/2-examples"},{"type":"type","title":"Scurry.Polygon.line/0","doc":"A line is represented by two vertices of type `t:vector/0`","ref":"Scurry.Polygon.html#t:line/0"},{"type":"function","title":"Scurry.Polygon.nearest_edge/2","doc":"Find the edge of a polygon nearest a given point.\n\nGiven a `point` that's inside or outside a given `polygon`, this checks each\nsegment of the polygon, and returns the nearest one.\n\nIf multiple lines are \"nearest\", the first in the polygon is picked.","ref":"Scurry.Polygon.html#nearest_edge/2"},{"type":"function","title":"Params - Scurry.Polygon.nearest_edge/2","doc":"* `polygon` (`t:polygon/0`) the polygion to find the nearest edge on.\n* `point` (`t:vector/0`) the point to find the nearest edge to.","ref":"Scurry.Polygon.html#nearest_edge/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.nearest_edge/2","doc":"A `t:line/0` that is a segment of `polygon` that is closest to `point`.","ref":"Scurry.Polygon.html#nearest_edge/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.nearest_edge/2","doc":"# A square around 0,0\n    iex> square = [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    iex> Polygon.nearest_edge(square, {0, 0})\n    {{-1, -1}, {1, -1}}\n    iex> Polygon.nearest_edge(square, {0.9, 0})\n    {{1, -1}, {1, 1}}\n    iex> Polygon.nearest_edge(square, {1.1, 0})\n    {{1, -1}, {1, 1}}","ref":"Scurry.Polygon.html#nearest_edge/2-examples"},{"type":"function","title":"Scurry.Polygon.nearest_point_on_edge/2","doc":"Find the point on the edge of a polygon nearest a given point.\n\nGiven a `point` that's inside or outside a given `polygon`, this uses\n`nearest_edge/2` to find the closest edge and then computes the point on the\nedge nearest the given `point`.","ref":"Scurry.Polygon.html#nearest_point_on_edge/2"},{"type":"function","title":"Params - Scurry.Polygon.nearest_point_on_edge/2","doc":"* `polygon` (`t:polygon/0`) the polygion to find the nearest point on.\n* `point` (`t:vector/0`) the point to find the nearest point to.","ref":"Scurry.Polygon.html#nearest_point_on_edge/2-params"},{"type":"function","title":"Returns - Scurry.Polygon.nearest_point_on_edge/2","doc":"A `t:vector/0` that is a point on `polygon` that is closest to `point`.","ref":"Scurry.Polygon.html#nearest_point_on_edge/2-returns"},{"type":"function","title":"Examples - Scurry.Polygon.nearest_point_on_edge/2","doc":"# A square around 0,0\n    iex> square = [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    [{-1, -1}, {1, -1}, {1, 1}, {-1, 1}]\n    iex> Polygon.nearest_point_on_edge(square, {0, 0})\n    {0.0, -1.0}\n    iex> Polygon.nearest_point_on_edge(square, {0.9, 0})\n    {1.0, 0.0}\n    iex> Polygon.nearest_point_on_edge(square, {1.1, 0})\n    {1.0, 0.0}","ref":"Scurry.Polygon.html#nearest_point_on_edge/2-examples"},{"type":"type","title":"Scurry.Polygon.polygon/0","doc":"A polygon is represented by a list of `t:vector/0`, each being a tuple of `{x, y}` coordinates, both `t:number/0`. To be properly defined, the vertices of the polygon must be non-closed and clockwise.","ref":"Scurry.Polygon.html#t:polygon/0"},{"type":"type","title":"Scurry.Polygon.vector/0","doc":"A vector is represented as a tuple `{x, y}`, it's x and y components, both `t:number/0`. These are used for vector math in `Scurry.Vector` and as 2D points in `Scurry.Polygon` and `Scurry.PolygonMap`.","ref":"Scurry.Polygon.html#t:vector/0"},{"type":"module","title":"Scurry.PolygonMap","doc":"Utility functions to work on a polygon map.\n\nA polygon map is a set of a primary polygon - the **main** boundary that\noutlines the world - and a list of polygons that make **holes** in the main\npolygon.\n\nSee `Polygon` for details on how polygons are composed.\n\nThe use case is eg. making a map (the **main** polygon) with obstacles (the\n**holes**), and use the `Astar` module to find the shortest path between\npoints in the map.\n\nSee [Quickstart](quickstart.html) for a concrete end-to-end example of\ndefining a world and holes/obstacles, then using `PolygonMap` and `Astar`\nmodules to do path finding within this.","ref":"Scurry.PolygonMap.html"},{"type":"type","title":"Scurry.PolygonMap.cost/0","doc":"The cost of a traversel between two nodes is a numeric value","ref":"Scurry.PolygonMap.html#t:cost/0"},{"type":"type","title":"Scurry.PolygonMap.cost_fun/0","doc":"Function that given two `t:gnode/0` graph nodes, computes the cost. Eg. a euclidian 2D vector function like `Scurry.Vector.distance/2`. `(gnode, gnode -> cost)`","ref":"Scurry.PolygonMap.html#t:cost_fun/0"},{"type":"function","title":"Scurry.PolygonMap.create_graph/4","doc":"Given a polygon map (`world` & `holes`) and list of vertices, makes the walk\ngraph for a A-star search.","ref":"Scurry.PolygonMap.html#create_graph/4"},{"type":"function","title":"Params - Scurry.PolygonMap.create_graph/4","doc":"* `world` (`t:polygon/0`) the polygon that defines the outer boundary.\n* `holes` (`t:list/0`), a list of `t:polygon/0` that define holes/obstacles inside `world`.\n* `vertices` (`t:list/0`), the result of `get_vertices/2`.\n* `cost_fun` (`t:cost_fun/0`) a \"cost\" function.\n\nThe `cost_fun` is a function that indicates the of traversing between to\npoints. It defaults to `Scurry.Vector.distance/2`, which is suitable for\nbasic 2D map walking.","ref":"Scurry.PolygonMap.html#create_graph/4-params"},{"type":"function","title":"Returns - Scurry.PolygonMap.create_graph/4","doc":"A `t:graph/0`, which is a map of all node to node reachable edges and their\ncost. This graphs is one of the parameters for a call to `Astar.search/4`.\n\n>","ref":"Scurry.PolygonMap.html#create_graph/4-returns"},{"type":"function","title":"Todo - Scurry.PolygonMap.create_graph/4","doc":">\n> This should ideally take `line_of_sight/3` as a function so users can\n> customise which vertices can reach each other. But for now, users can make\n> the graph themselves just as easily.","ref":"Scurry.PolygonMap.html#create_graph/4-todo"},{"type":"function","title":"Scurry.PolygonMap.extend_graph/6","doc":"Given a polygon map (main & holes), list of vertices and the initial graph\nfrom `create_graph/4`, extend the graph with extra `points`.\n\nThis is used to \"temporarily\" expand the created walk graph with a start and\nend-point. This is a minor performance optimisation that saves work by\nreusing the fixed nodes and extend it with the moveable points.","ref":"Scurry.PolygonMap.html#extend_graph/6"},{"type":"function","title":"Params - Scurry.PolygonMap.extend_graph/6","doc":"* `graph` (`t:graph/0`) the walk graph, as created by `create_graph/4`. This is used to return a new graph that's extended with `points`.\n* `world` (`t:polygon/0`) the polygon that defines the outer boundary.\n* `holes` (`t:list/0`), a list of `t:polygon/0` that define holes/obstacles inside `world`.\n* `vertices` (`t:list/0`) the list of vertices returned by `get_vertices/4`\n  and used for `create_graph/4` to get `graph`.\n* `points` (`t:list/0`) a list of vectors (`t:vector/0`), `[{x, y}, {x, y}...]`, to extend with. Eg. a stop\n* `cost_fun` (`t:cost_fun/0`) a \"cost\" function.\n\n`world` and `holes` need to be passed in in addition to `graph`, so the\nfunction can determine line-of-sights between the existing graph and new\npoints from `points`.\n\nThe `cost_fun` is a function that indicates the of traversing between to\npoints. It defaults to `Scurry.Vector.distance/2`, which is suitable for\nbasic 2D map walking.","ref":"Scurry.PolygonMap.html#extend_graph/6-params"},{"type":"function","title":"Returns - Scurry.PolygonMap.extend_graph/6","doc":"A tuple with the extended graph plus the combined list of vertices and new\npoints, `{new_graph, new_vertices}`. The `new_graph` is intended to be used\nin calls to `Astar.search/4`, and `new_vertices` is solely provided as\ninformational, eg. for displaying the graphs visually.","ref":"Scurry.PolygonMap.html#extend_graph/6-returns"},{"type":"function","title":"Scurry.PolygonMap.get_vertices/2","doc":"Given a polygon map (`world` & `holes`), returns a list of vertices for a\nwalkmap for A-star search","ref":"Scurry.PolygonMap.html#get_vertices/2"},{"type":"function","title":"Params - Scurry.PolygonMap.get_vertices/2","doc":"* `world` (`t:polygon/0`), the polygon that defines the outer boundary.\n* `holes` (`t:list/0`), a list of `t:polygon/0` that define holes/obstacles inside `world`.","ref":"Scurry.PolygonMap.html#get_vertices/2-params"},{"type":"function","title":"Returns - Scurry.PolygonMap.get_vertices/2","doc":"The walkmap as a `t:list/0` of `t:vector/0` that are the `worlds`'s concave\nvertices and the convex ones of the `holes`.\n\nThese are used when generating the walk map, since only the `world`'s concave\nand the `holes`' convex ones limit where you can traverse in a 2D map. Any\npath has to walk around the holes' convex points and the world's concave.\n\nSee [Quickstart](quickstart.html) for a concrete example.","ref":"Scurry.PolygonMap.html#get_vertices/2-returns"},{"type":"type","title":"Scurry.PolygonMap.gnode/0","doc":"A graph node (spelled `gnode` since `node` is reserved) is any type that can be used as a key in a `t:map/0` For instance a `t:vector/0`.","ref":"Scurry.PolygonMap.html#t:gnode/0"},{"type":"type","title":"Scurry.PolygonMap.graph/0","doc":"A graph is a map from `t:gnode/0` to a list of `t:gnode/0` and the traversal `t:cost/0`, `%{gnode => [{gnode, cost}, ...]}`","ref":"Scurry.PolygonMap.html#t:graph/0"},{"type":"function","title":"Scurry.PolygonMap.is_line_of_sight?/3","doc":"Checks if there's a line-of-sight (LOS) from `start` to `stop` within the map.","ref":"Scurry.PolygonMap.html#is_line_of_sight?/3"},{"type":"function","title":"Params - Scurry.PolygonMap.is_line_of_sight?/3","doc":"* `polygon`, a list of `{x, y}` vertices. This is the main boundary map.\n* `holes`, a list of lists of `{x, y}` vertices. These are holes within\n  `polygon`.\n* `line` a tuple of points (`{{ax, ay}, {bx, by}}`) describing a line.\n\nReturns `true` if there's a line-of-sight and none of the main polygon or\nholes obstruct the path. `false` otherwise.\n\nAs the map consists of a boundary polygon with holes, LOS implies a few things;\n\n* If either `start` or `stop` is outside `polygon`, the result will be\n  false. Even if both are outside, that's not considered a valid LOS.\n* If the distance between `start` and `stop` is tiny (< 0.1 arbitrarily), LOS\n  is true.\n* Next, it checks that the line between `start` and `stop` has no\n  intersections with `polygon` or `holes`.\n* Finally it checks if the middle of the line between `start` and `stop` is\n  inside `polygon` and outside all holes - this ensures that corner-to-corner\n  across a hole isn't considered a LOS.","ref":"Scurry.PolygonMap.html#is_line_of_sight?/3-params"},{"type":"type","title":"Scurry.PolygonMap.line/0","doc":"A line is represented by two vertices of type `t:vector/0`","ref":"Scurry.PolygonMap.html#t:line/0"},{"type":"function","title":"Scurry.PolygonMap.nearest_point/3","doc":"Find the nearest point on the line that is inside the world polygion and\noutside all hole polygons.\n\nThe purpose of this function is to determine compute a suitable end for the\n`points` parameter for `extend_graph/6`. Imagine a UI in which the users\nclicks on a point to walk to. If that click is outside the world map or\ninside a hole/obstacle, we can either refuse to compute a path, or we can use\n`nearest_point/3` to find the closest reachable point to the click.","ref":"Scurry.PolygonMap.html#nearest_point/3"},{"type":"function","title":"Params - Scurry.PolygonMap.nearest_point/3","doc":"* `world` (`t:polygon/0`) the polygon that defines the outer boundary.\n* `holes` (`t:list/0`), a list of `t:polygon/0` that define holes/obstacles inside `world`.\n* `point` (`t:vector/0`) the point for which to find the nearst point.","ref":"Scurry.PolygonMap.html#nearest_point/3-params"},{"type":"function","title":"Returns - Scurry.PolygonMap.nearest_point/3","doc":"A new point (`t:vector/0`) `p={x, y}` such that;\n\n* if `point` is _outside_ the world polygon, the new `p` is the closest point *on*\n  the edge of the `world` polygon.\n\n* if `point` is _inside_ the world polygon, but also *inside* any hole\npolygon, the `p` the closest point on the edge of the hole it's in.","ref":"Scurry.PolygonMap.html#nearest_point/3-returns"},{"type":"type","title":"Scurry.PolygonMap.polygon/0","doc":"A polygon is represented by a list of `t:vector/0`, each being a tuple of `{x, y}` coordinates, both `t:number/0`. To be properly defined, the vertices of the polygon must be non-closed and clockwise.","ref":"Scurry.PolygonMap.html#t:polygon/0"},{"type":"type","title":"Scurry.PolygonMap.vector/0","doc":"A vector is represented as a tuple `{x, y}`, it's x and y components, both `t:number/0`. These are used for vector math in `Scurry.Vector` and as 2D points in `Scurry.Polygon` and `Scurry.PolygonMap`.","ref":"Scurry.PolygonMap.html#t:vector/0"},{"type":"module","title":"Scurry.Vector","doc":"Functions to work on 2D vectors.\n\nVectors are represented as tuples of x and y components, `{x :: number, y :: number}`. This\nmodule provides basic trigonometry functions.\n\nSee [Euclidian Vector](https://en.wikipedia.org/wiki/Euclidean_vector) on\nWikipedia for further descriptions of the maths and use cases.","ref":"Scurry.Vector.html"},{"type":"function","title":"Scurry.Vector.add/2","doc":"Add two vectors together.","ref":"Scurry.Vector.html#add/2"},{"type":"function","title":"Params - Scurry.Vector.add/2","doc":"* `v1` (`t:vector/0`) the first vector.\n* `v2` (`t:vector/0`) the second vector to add to `v1`.","ref":"Scurry.Vector.html#add/2-params"},{"type":"function","title":"Returns - Scurry.Vector.add/2","doc":"The resulting vector of adding `v1` and `v2` together.","ref":"Scurry.Vector.html#add/2-returns"},{"type":"function","title":"Examples - Scurry.Vector.add/2","doc":"iex> Vector.add({1, 2}, {3, 4})\n    {4, 6}","ref":"Scurry.Vector.html#add/2-examples"},{"type":"function","title":"Scurry.Vector.angle/1","doc":"Get the angle of a vector in radians.","ref":"Scurry.Vector.html#angle/1"},{"type":"function","title":"Params - Scurry.Vector.angle/1","doc":"* `v` (`t:vector/0`) describing the vector to obtain the angle for.","ref":"Scurry.Vector.html#angle/1-params"},{"type":"function","title":"Returns - Scurry.Vector.angle/1","doc":"The angle in radians in relationship to the x-axis.","ref":"Scurry.Vector.html#angle/1-returns"},{"type":"function","title":"Examples - Scurry.Vector.angle/1","doc":"iex> Vector.angle({1, 1})\n    0.7853981633974483\n    iex> Vector.angle({0, 1})\n    1.5707963267948966\n    iex> Vector.angle({1, 0})\n    0.0\n    iex> Vector.angle({-1, 1})\n    2.356194490192345\n    iex> Vector.angle({-1, 0})\n    3.141592653589793\n    iex> Vector.angle({-1, -1})\n    3.9269908169872414\n    iex> Vector.angle({0, -1})\n    4.71238898038469\n    iex> Vector.angle({1, -1})\n    5.497787143782138","ref":"Scurry.Vector.html#angle/1-examples"},{"type":"function","title":"Scurry.Vector.cross/2","doc":"Get the cross product of two vectors.","ref":"Scurry.Vector.html#cross/2"},{"type":"function","title":"Params - Scurry.Vector.cross/2","doc":"* `v1` (`t:vector/0`) describing the first vector.\n* `v2` (`t:vector/0`) describing the second vector to 'cross' against `v2`.\n\nReturns the cross product of `v1` and `v2`, `v1` × `v2`.","ref":"Scurry.Vector.html#cross/2-params"},{"type":"function","title":"Examples - Scurry.Vector.cross/2","doc":"iex> Vector.cross({1, 2}, {3, 4})\n    -2","ref":"Scurry.Vector.html#cross/2-examples"},{"type":"function","title":"Scurry.Vector.degrees/1","doc":"This is a screen oriented degree.\n\nScreen degrees are in\n* North=up (0)\n* South=down (180) degrees.\n\nVectors are in `(x,y)` screen coordinate, so `(1,1)` = 135 degrees (down to the\nright/ south-east) from `0,0`, the top left corner is `0,0`.\n\nThis layout maps how eg. [`:wx`](https://www.erlang.org/doc/man/wx.html)\ndefines the coordinates.","ref":"Scurry.Vector.html#degrees/1"},{"type":"function","title":"Params - Scurry.Vector.degrees/1","doc":"* `v` (`t:vector/0`) of coordinates describing the vector to obtain the angle for.","ref":"Scurry.Vector.html#degrees/1-params"},{"type":"function","title":"Returns - Scurry.Vector.degrees/1","doc":"The vector in degrees relative to the y-axis.","ref":"Scurry.Vector.html#degrees/1-returns"},{"type":"function","title":"Examples - Scurry.Vector.degrees/1","doc":"iex> Vector.degrees({0, -1})\n    0\n    iex> Vector.degrees({1, -1})\n    45\n    iex> Vector.degrees({1, 0})\n    90\n    iex> Vector.degrees({1, 1})\n    135\n    iex> Vector.degrees({0, 1})\n    180\n    iex> Vector.degrees({-1, 1})\n    225\n    iex> Vector.degrees({-1, 0})\n    270\n    iex> Vector.degrees({-1, -1})\n    315","ref":"Scurry.Vector.html#degrees/1-examples"},{"type":"function","title":"Scurry.Vector.degrees_graph/1","doc":"This is a graph oriented degree.\n\nGraph degrees are oriented as for a graph layout.\n\n* Right (along the x-axis) is 0°\n* Up (along y-axis) is 90°,\n* 45° is \"north-east / up and to the left\".","ref":"Scurry.Vector.html#degrees_graph/1"},{"type":"function","title":"Params - Scurry.Vector.degrees_graph/1","doc":"* `v` (`t:vector/0`) describing the vector to obtain the angle for.","ref":"Scurry.Vector.html#degrees_graph/1-params"},{"type":"function","title":"Returns - Scurry.Vector.degrees_graph/1","doc":"The vector in degrees relative to the x-axis.","ref":"Scurry.Vector.html#degrees_graph/1-returns"},{"type":"function","title":"Examples - Scurry.Vector.degrees_graph/1","doc":"iex> Vector.degrees_graph({1, 1})\n    45.0\n    iex> Vector.degrees_graph({0, 1})\n    90.0\n    iex> Vector.degrees_graph({1, 0})\n    0.0\n    iex> Vector.degrees_graph({-1, 1})\n    135.0\n    iex> Vector.degrees_graph({-1, 0})\n    180.0\n    iex> Vector.degrees_graph({-1, -1})\n    225.0\n    iex> Vector.degrees_graph({0, -1})\n    270.0\n    iex> Vector.degrees_graph({1, -1})\n    315.0","ref":"Scurry.Vector.html#degrees_graph/1-examples"},{"type":"function","title":"Scurry.Vector.distance/2","doc":"Get the distance of two vectors.\n\nThe distance is the length of the vector between the ends (second tuple) of the vectors.\n\nThis is equivalent to the square root of the `distance_squared/2`.","ref":"Scurry.Vector.html#distance/2"},{"type":"function","title":"Params - Scurry.Vector.distance/2","doc":"* `v1` (`t:vector/0`) describing the first vector.\n* `v2` (`t:vector/0`) describing the second vector to get the distance from `v1`.","ref":"Scurry.Vector.html#distance/2-params"},{"type":"function","title":"Returns - Scurry.Vector.distance/2","doc":"The distance between the ends of `v1` and `v2`.","ref":"Scurry.Vector.html#distance/2-returns"},{"type":"function","title":"Examples - Scurry.Vector.distance/2","doc":"iex> Vector.distance({0, 0}, {1, 1})\n    1.4142135623730951\n    iex> Vector.distance({5, 7}, {2, 3})\n    5.0","ref":"Scurry.Vector.html#distance/2-examples"},{"type":"function","title":"Scurry.Vector.distance_squared/2","doc":"Get the distance squared of two vectors.\n\nThis is equivalent to the square of the `distance/2`.","ref":"Scurry.Vector.html#distance_squared/2"},{"type":"function","title":"Params - Scurry.Vector.distance_squared/2","doc":"* `v1` (`t:vector/0`) describing the first vector.\n* `v2` (`t:vector/0`) describing the second vector to get the distance squared from `v1`.","ref":"Scurry.Vector.html#distance_squared/2-params"},{"type":"function","title":"Returns - Scurry.Vector.distance_squared/2","doc":"The squared distance between the ends of `v1` and `v2`.","ref":"Scurry.Vector.html#distance_squared/2-returns"},{"type":"function","title":"Examples - Scurry.Vector.distance_squared/2","doc":"iex> Vector.distance_squared({0, 0}, {1, 1})\n    2.0\n    iex> Vector.distance_squared({5, 7}, {2, 3})\n    25.0","ref":"Scurry.Vector.html#distance_squared/2-examples"},{"type":"function","title":"Scurry.Vector.div/2","doc":"Divide a vector by a constant.\n\nDividing/multiplying a vector essentially shortens/lengthens it.","ref":"Scurry.Vector.html#div/2"},{"type":"function","title":"Params - Scurry.Vector.div/2","doc":"* `v` (`t:vector/0`) the vector to divide.\n* `c` (`t:number/0`) the constant to divide `v` by.","ref":"Scurry.Vector.html#div/2-params"},{"type":"function","title":"Returns - Scurry.Vector.div/2","doc":"The result of dividing `v` by `c`.","ref":"Scurry.Vector.html#div/2-returns"},{"type":"function","title":"Examples - Scurry.Vector.div/2","doc":"iex> Vector.div({10, 14}, 2)\n    {5.0, 7.0}","ref":"Scurry.Vector.html#div/2-examples"},{"type":"function","title":"Scurry.Vector.dot/2","doc":"Get the dot product of two vectors.","ref":"Scurry.Vector.html#dot/2"},{"type":"function","title":"Params - Scurry.Vector.dot/2","doc":"* `v1` (`t:vector/0`) describing the first vector.\n* `v2` (`t:vector/0`) describing the second vector to 'dot' against `v1`.","ref":"Scurry.Vector.html#dot/2-params"},{"type":"function","title":"Returns - Scurry.Vector.dot/2","doc":"The dot product of `v1` and `v2`, `v1` · `v2`.","ref":"Scurry.Vector.html#dot/2-returns"},{"type":"function","title":"Examples - Scurry.Vector.dot/2","doc":"iex> Vector.dot({1, 2}, {3, 4})\n    11","ref":"Scurry.Vector.html#dot/2-examples"},{"type":"function","title":"Scurry.Vector.len/1","doc":"Get the length of a vector, aka magnitude.","ref":"Scurry.Vector.html#len/1"},{"type":"function","title":"Params - Scurry.Vector.len/1","doc":"* `v` (`t:vector/0`) the vector to find the length for.","ref":"Scurry.Vector.html#len/1-params"},{"type":"function","title":"Returns - Scurry.Vector.len/1","doc":"The length of the vector `v`.","ref":"Scurry.Vector.html#len/1-returns"},{"type":"function","title":"Examples - Scurry.Vector.len/1","doc":"iex> Vector.len({1, 1})\n    1.4142135623730951\n    iex> Vector.len({3, 4})\n    5.0\n    iex> Vector.len({12, 5})\n    13.0","ref":"Scurry.Vector.html#len/1-examples"},{"type":"type","title":"Scurry.Vector.line/0","doc":"A line is represented by two vertices of type `t:vector/0`","ref":"Scurry.Vector.html#t:line/0"},{"type":"function","title":"Scurry.Vector.mag/1","doc":"Get the magnitude of a vector, aka len.\n\nThis is an alias for `len/2`.","ref":"Scurry.Vector.html#mag/1"},{"type":"function","title":"Examples - Scurry.Vector.mag/1","doc":"iex> Vector.mag({1, 1})\n    1.4142135623730951\n    iex> Vector.mag({3, 4})\n    5.0\n    iex> Vector.mag({12, 5})\n    13.0","ref":"Scurry.Vector.html#mag/1-examples"},{"type":"function","title":"Scurry.Vector.mul/2","doc":"Multiply a vector by a constant.","ref":"Scurry.Vector.html#mul/2"},{"type":"function","title":"Params - Scurry.Vector.mul/2","doc":"* `v` (`t:vector/0`) describing the vector.\n* `c` (`t:number/0` the constant to multiply `v` by.\n\nReturns the result of multiplying `v` by `c`.","ref":"Scurry.Vector.html#mul/2-params"},{"type":"function","title":"Examples - Scurry.Vector.mul/2","doc":"iex> Vector.mul({5, 7}, 2)\n    {10, 14}","ref":"Scurry.Vector.html#mul/2-examples"},{"type":"function","title":"Scurry.Vector.normalise/1","doc":"Normalise a vector to length 1.\n\nThis shortens the vector by it's length, so the resulting vector `w` has\n`len(w) == 1`, and same x/y ratio.","ref":"Scurry.Vector.html#normalise/1"},{"type":"function","title":"Params - Scurry.Vector.normalise/1","doc":"* `v` (`t:vector/0`) describing the vector to normalise.","ref":"Scurry.Vector.html#normalise/1-params"},{"type":"function","title":"Returns - Scurry.Vector.normalise/1","doc":"A `t:vector/0` with length 1, and same x/y ratio.","ref":"Scurry.Vector.html#normalise/1-returns"},{"type":"function","title":"Examples - Scurry.Vector.normalise/1","doc":"iex> Vector.normalise({0, 1})\n    {0.0, 1.0}\n    iex> Vector.normalise({10, 0})\n    {1.0, 0.0}\n    iex> Vector.normalise({10, 10})\n    {0.7071067811865475, 0.7071067811865475}","ref":"Scurry.Vector.html#normalise/1-examples"},{"type":"type","title":"Scurry.Vector.polygon/0","doc":"A polygon is represented by a list of `t:vector/0`, each being a tuple of `{x, y}` coordinates, both `t:number/0`. To be properly defined, the vertices of the polygon must be non-closed and clockwise.","ref":"Scurry.Vector.html#t:polygon/0"},{"type":"function","title":"Scurry.Vector.round_pos/1","doc":"Calls round on a vector to make a vector with `t:integer/0` instead of `t:float/0`.\n\nThis is for interoperability with other libraries where coordinates must be\nexpressed in integers, for example\n[`:wx`](https://www.erlang.org/doc/man/wx.html) operations for drawing.\n\nThe name ends in `_pos` to avoid any confusion/collision with\n`Kernel.round/1` and to indicate the use in drawing \"positions\" on the\nscreen.","ref":"Scurry.Vector.html#round_pos/1"},{"type":"function","title":"Params - Scurry.Vector.round_pos/1","doc":"* `v` (`t:vector/0`) describing the vector to round to integers.","ref":"Scurry.Vector.html#round_pos/1-params"},{"type":"function","title":"Returns - Scurry.Vector.round_pos/1","doc":"Avector with it's components converted to integers using `Kernel.round/1`.","ref":"Scurry.Vector.html#round_pos/1-returns"},{"type":"function","title":"Examples - Scurry.Vector.round_pos/1","doc":"iex> Vector.round_pos({10.1, 10.9})\n    {10, 11}","ref":"Scurry.Vector.html#round_pos/1-examples"},{"type":"function","title":"Scurry.Vector.sub/2","doc":"Subtract a vector from another.","ref":"Scurry.Vector.html#sub/2"},{"type":"function","title":"Params - Scurry.Vector.sub/2","doc":"* `v1` (`t:vector/0`) the first vector.\n* `v2` (`t:vector/0`) the second vector to subtract from `v1`.","ref":"Scurry.Vector.html#sub/2-params"},{"type":"function","title":"Returns - Scurry.Vector.sub/2","doc":"The resulting vector of subtracting of `v2` from `v1`.","ref":"Scurry.Vector.html#sub/2-returns"},{"type":"function","title":"Examples - Scurry.Vector.sub/2","doc":"iex> Vector.sub({5, 7}, {1, 2})\n    {4, 5}","ref":"Scurry.Vector.html#sub/2-examples"},{"type":"function","title":"Scurry.Vector.trunc_pos/1","doc":"Calls round on a vector to make a vector with `t:integer/0` instead of `t:float/0`.\n\nThis is provided for interoperability with other libraries where coordinates\nmust be expressed in integers, for example\n[`:wx`](https://www.erlang.org/doc/man/wx.html) operations for drawing.\n\nThe name ends in `_pos` to avoid any confusion/collision with\n`Kernel.trunc/1` and to indicate the use in drawing \"positions\" on the\nscreen.","ref":"Scurry.Vector.html#trunc_pos/1"},{"type":"function","title":"Params - Scurry.Vector.trunc_pos/1","doc":"* `v` (`t:vector/0`) describing the vector to round to integers.","ref":"Scurry.Vector.html#trunc_pos/1-params"},{"type":"function","title":"Returns - Scurry.Vector.trunc_pos/1","doc":"The vector with it's components converted to integers using `Kernel.trunc/1`.","ref":"Scurry.Vector.html#trunc_pos/1-returns"},{"type":"function","title":"Examples - Scurry.Vector.trunc_pos/1","doc":"iex> Vector.trunc_pos({10.1, 10.9})\n    {10, 10}","ref":"Scurry.Vector.html#trunc_pos/1-examples"},{"type":"type","title":"Scurry.Vector.vector/0","doc":"A vector is represented as a tuple `{x, y}`, it's x and y components, both `t:number/0`. These are used for vector math in `Scurry.Vector` and as 2D points in `Scurry.Polygon` and `Scurry.PolygonMap`.","ref":"Scurry.Vector.html#t:vector/0"},{"type":"extras","title":"Quickstart","doc":"# Quickstart","ref":"quickstart.html"},{"type":"extras","title":"Search a 2D Polygon Map - Quickstart","doc":"There's four steps to do a 2D polygon map path search\n\n1. Create a polygon map with holes\n1. Create a graph from the map\n1. Extend the graph with your start and stop\n1. Use Astar to find a path\n\nYou can try out the code below in iex.\n\n```shell\niex -S mix\n```\n\n```elixir\nalias Scurry.PolygonMap\nalias Scurry.Vector\nalias Scurry.Astar","ref":"quickstart.html#search-a-2d-polygon-map"},{"type":"extras","title":"Step 1 - define the world - Quickstart","doc":"# The world is defined by a boundary box plus holes inside the box.\n\n# World boundary\npolygon = [{0, 0}, {100, 0}, {100, 100}, {0, 100}]\n\n# Two holes\nholes = [\n [{30, 30}, {40, 30}, {40, 70}, {30, 70}],\n [{60, 30}, {70, 30}, {70, 70}, {60, 70}],\n]","ref":"quickstart.html#step-1-define-the-world"},{"type":"extras","title":"Step 2 - convert world to graph - Quickstart","doc":"# To compute a path, we need to convert the boundary & holes into\n# a graph of all corners.\n\n# The cost function for the graph, this uses basic euclidian distance\n# as a basic example. This is used to compute the graph of the world.\ncost_fun = fn a, b -> Vector.distance(a, b) end\n\n# Find the reachable vertices\nvertices = PolygonMap.get_vertices(polygon, holes)\n\n# Make a graph of them using an optional cost_fun\ngraph = PolygonMap.create_graph(polygon, holes, vertices, cost_fun)","ref":"quickstart.html#step-2-convert-world-to-graph"},{"type":"extras","title":"Step 3 - extend graph with actor start/stop - Quickstart","doc":"# For an actor to move from a to b in the world, we first extend the\n# graph with start and stop\n\n# Define our start & stop\n{start, stop} = {{10, 10}, {90, 90}}\n\n# Create the temporary extended graph with start & stop and an optional cost_fun\n{search_graph, _} =\n  PolygonMap.extend_graph(graph, polygon, holes,\n  vertices, [start, stop], cost_fun)","ref":"quickstart.html#step-3-extend-graph-with-actor-start-stop"},{"type":"extras","title":"Step 4 - Quickstart","doc":"# Given the new graph, finally compute the path in the world.\n\n# The heuristic function for A-star, this uses euclidian distance as well.\n# This is used as the cost of each edge traversal in the graph during a-star\n# path finding. It can be the same as cost_fun but it does not have to be.\nheur_fun = fn a, b -> Vector.distance(a, b) end\n\n# Astar search\nastar = Astar.search(search_graph, start, stop, heur_fun)\npath = Astar.path(astar)\n\n# Result\n[{10, 10}, {40, 30}, {60, 70}, {90, 90}]\n```\n\n![Sample](imgs/quickstart.png)","ref":"quickstart.html#step-4"},{"type":"extras","title":"Search a adhoc graph - Quickstart","doc":"Very similar to an 2D polygon map. A graph doens't have to be screen\ncoordinates, just terms that can be map keys.\n\n1. Create a graph\n1. Use Astar to find a path\n\n```elixier\nalias Scurry.Astar","ref":"quickstart.html#search-a-adhoc-graph"},{"type":"extras","title":"Step 1 - Quickstart","doc":"# A graph is a map from nodes to a list of edges, an edge is a {node,\n# cost} tuple.\n\ngraph = fn ->\n  %{\n    \"a\" => [{\"b\", 4}, {\"c\", 3}],\n    \"b\" => [{\"f\", 5}, {\"e\", 12}],\n    \"c\" => [{\"e\", 10}, {\"d\", 7}],\n    \"d\" => [{\"e\", 2}],\n    \"e\" => [{\"z\", 5}],\n    \"f\" => [{\"z\", 16}],\n  }\nend\n\n# The heuristic function takes two nodes and returns an estimated\n# cost. It will be called with the destination node passed to A-star.\n\nheur = fn from, to ->\n  %{\n    \"a\" => %{\"z\" => 14},\n    \"b\" => %{\"z\" => 12},\n    \"c\" => %{\"z\" => 11},\n    \"d\" => %{\"z\" => 6},\n    \"e\" => %{\"z\" => 4},\n    \"f\" => %{\"z\" => 11},\n    \"z\" => %{\"z\" => 0},\n  }[from][to]\nend","ref":"quickstart.html#step-1"},{"type":"extras","title":"Step 2 - Quickstart","doc":"# Astar search\n\nstate = Astar.search(graph.(), \"a\", \"z\", heur)\npath = Astar.path(state)\n\n# Result\n[\"a\", \"c\", \"d\", \"e\", \"z\"]\n```","ref":"quickstart.html#step-2"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}